package evenosmaster.util;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Iterator;
import java.util.Timer;
import java.util.concurrent.TimeUnit;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class Main {

	public static void main(String[] args) throws NumberFormatException, Exception {

		// Parse command line arguments
		CommandLineParser clp = new GnuParser();
		Options options = createOpptions();
		CommandLine cl = null;
		try {
			cl = clp.parse(options, args);
		} catch (ParseException e) {
			printHelp(options);
			System.exit(-1);
		}

		// Tell the user which arguments is used
		// Enable user to discover misspelled arguments
		System.out.println("Running with the following options:");
		@SuppressWarnings("unchecked")
		Iterator<Option> it = cl.iterator();
		if (it.hasNext()) {
			System.out.print(it.next().getOpt());
		}
		while (it.hasNext()) {
			System.out.print(" " + it.next().getOpt());
		}
		System.out.println();

		if (cl.hasOption("help")) {
			printHelp(options);
			System.exit(0);
		}

		// Parse arguments
		final String host = cl.getOptionValue("host", "127.0.0.1");
		final int port = Integer.parseInt(cl.getOptionValue("port", "27019"));
		final String folder = cl.getOptionValue("folder");
		final int totalWorkers = Integer.parseInt(cl.getOptionValue("totalWorkers", "10"));
		final int localWorkers = Integer.parseInt(cl.getOptionValue("localWorkers", "10"));
		final int localRangeStart = Integer.parseInt(cl.getOptionValue("localRangeStart", "0"));
		final boolean useShards = cl.hasOption("useShards");
		final String statsFile = cl.getOptionValue("statsFile");

		final MongoLoade ls = new MongoLoade(host, port, useShards, localWorkers);

		performTest(folder, totalWorkers, localWorkers, localRangeStart, useShards, statsFile, ls);

	}

	private static Options createOpptions() {
		Options options = new Options();
	
		Option host = new Option("h", "host", true, "address of mongoDB");
		Option port = new Option("p", "port", true, "the port of the mongoDB servr");
	
		Option folder = new Option("f", "folder", true, "the folder containing the documents to be loaded");
		folder.setRequired(true);
	
		Option totalWorkers = new Option("t", "totalWorkers", true, "the global total number of workers loading data");
		Option localWorkers = new Option("l", "localWorkers", true, "the number of local workers loading data");
		Option localRangeStart = new Option("r", "localRangeStart", true,
				"the number of documents to skipp at the start");
	
		Option useShards = new Option("s", "useShards", false, "Shuld the server use shards true/false");
	
		Option help = new Option("help", false, "print help, this section");
	
		Option statsFile = new Option("sf", "statsFile", true, "file where the statistics wil be saved");
	
		options.addOption(host);
		options.addOption(port);
		options.addOption(folder);
		options.addOption(totalWorkers);
		options.addOption(localWorkers);
		options.addOption(localRangeStart);
		options.addOption(useShards);
		options.addOption(help);
		options.addOption(statsFile);
	
		return options;
	}

	private static void printHelp(Options options) {
		HelpFormatter formatter = new HelpFormatter();
		formatter.printHelp("-f folder -sf statisticsFile [...]", options);
	}

	private static void performTest(final String folder, final int totalWorkers, final int localWorkers,
			final int localRangeStart, final boolean useShards, final String statsFile, final MongoLoade ls)
			throws IOException, InterruptedException {

		final DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
		final Calendar cal = Calendar.getInstance();

		// Print stats every 5 seconds
		Timer timer = new Timer(true);
		StatCollectorTask sct = new StatCollectorTask(ls, new File(statsFile));
		timer.scheduleAtFixedRate(sct, 5000, 5000);

		System.out.println("Starting loading of data at " + dateFormat.format(cal.getTime()));

		final long startTime = System.nanoTime();
		for(int i = 0; i < 21; i++)
			ls.addLoadTasks(folder, totalWorkers, localWorkers, localRangeStart, useShards);
		ls.awaitCompeltion(12, TimeUnit.HOURS);
		final long completionTime = System.nanoTime() - startTime;

		sct.close();
		timer.cancel();

		System.out.println("Loading completed at " + dateFormat.format(cal.getTime()));
		System.out.println("The prosess took " + completionTime + "ns");
		System.out.println("Final stats: " + ls.getStats());
	}

}
