package evenosmaster.util;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public class MongoLoade {
	
	private final String databaseName = "evenosmaster";
	private final MongoDBLoader mongoClient;
	private final ExecutorService executorService;
	private final StatisticsCollector stats;
	
	public MongoLoade(String host, int port, boolean useShard, int numThreads) throws Exception {
		stats = new StatisticsCollector();

		mongoClient = new MongoDBLoader(host, port, databaseName, stats);
		executorService = Executors.newFixedThreadPool(numThreads);
	}
	
	public void awaitCompeltion(long timeout, TimeUnit unit) throws InterruptedException {
		executorService.awaitTermination(timeout, unit);
	}

	public void addLoadTasks(final String folder, final int totalWorkers, final int localWorkers,
			final int localRangeStart, boolean useShards) throws IOException, InterruptedException {

		for (Path file : Files.newDirectoryStream(Paths.get(folder), "*.json")) {
			String name = file.getFileName().toString();
			int fileNumber = Integer.parseInt(name.substring(2, name.length() - 5));
			int test = fileNumber % totalWorkers;
			if (test >= localRangeStart && test < localRangeStart + localWorkers) {
				// MongoClient is thread safe
				executorService.execute(new FileInserter(file, mongoClient));
			}
		}
	}
	
	public Statistics getStats() {
		return stats.getCurrentStatistics();
	}

}
