package evenosmaster.util.xmltojson;

import java.io.IOException;
import java.util.Stack;

import org.json.JSONObject;
import org.json.XML;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

public class EvenosmasterHandeler extends DefaultHandler {
	private Stack<String> tags;
	private StringBuilder xml;
	private ObjectHandler<String> oh;

	public EvenosmasterHandeler(ObjectHandler<String> oh) {
		tags = new Stack<String>();
		this.oh = oh;
	}

	@Override
	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		tags.push(qName.toLowerCase());
		if (tags.peek().equalsIgnoreCase("revision")) {
			xml = new StringBuilder();
		}
		if (xml != null)
			xml.append("<" + qName + ">");

	}

	@Override
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		if (tags.peek().equalsIgnoreCase(qName)) { // Should always be true,
													// given correct XML
			tags.pop();
		}
		if (xml != null) {
			xml.append("</" + qName + ">");
			if (qName.equalsIgnoreCase("revision")) {
				String xmlString = xml.toString();
				JSONObject obj = XML.toJSONObject(xmlString);
				oh.handleObject(obj.toString());
			}
		}

	}

	@Override
	public void characters(char[] ch, int start, int length)
			throws SAXException {
		String text = new String(ch, start, length).trim();
		if(tags.peek().equalsIgnoreCase("title")){
			try {
				oh.collection(text);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}else if (xml != null) {
			// Escape all characters to avoid conflicts during conversion to
			// JSON
			if(tags.peek().equalsIgnoreCase("text") && xml.length() >= (Integer.MAX_VALUE * 0.85)){
				System.out.println("Skipping rest of to long string");
				return;
			}else{
				xml.append(
						XML.escape(text) );
			}
		}

	}

}
